@@ -34,12 +34,15 @@ def clean_val(val):
 
 # Same as above, except this version returns float('nan') when it fails
 # This plays more nicely with df.apply, and assumes we will be handling nans appropriately when doing DataFrameVectorizer later.
-def clean_val_nan_version(val):
+def clean_val_nan_version(key, val):
     try:
         str_val = str(val)
     except UnicodeEncodeError as e:
+        str_val = val.encode('ascii', 'ignore').decode('ascii')
         print('Here is the value that causes the UnicodeEncodeError to be thrown:')
         print(val)
+        print('Here is the feature name:')
+        print(key)
         raise(e)
 
     if str_val in bad_vals_as_strings:
@@ -48,7 +51,6 @@ def clean_val_nan_version(val):
         try:
             float_val = float(val)
         except ValueError:
-            # This will throw a ValueError if it fails
             # remove any commas in the string, and try to turn into a float again
             try:
                 cleaned_string = val.replace(',', '')
@@ -56,12 +58,24 @@ def clean_val_nan_version(val):
                 print('*************************************')
                 print('We expected this value to be numeric, but were unable to convert it to a float:')
                 print(val)
+                print('Here is the feature name:')
+                print(key)
                 print('*************************************')
                 return float('nan')
             try:
                 float_val = float(cleaned_string)
             except:
                 return float('nan')
+        except TypeError:
+            # This is what happens if you feed in a datetime object to float
+            print('*************************************')
+            print('We expected this value to be numeric, but were unable to convert it to a float:')
+            print(val)
+            print('Here is the feature name:')
+            print(key)
+            print('*************************************')
+            return float('nan')
+
         return float_val
 
 
@@ -71,6 +85,7 @@ class BasicDataCleaning(BaseEstimator, TransformerMixin):
 
     def __init__(self, column_descriptions=None):
         self.column_descriptions = column_descriptions
+        self.transformed_column_descriptions = column_descriptions.copy()
         self.text_col_indicators = set(['text', 'nlp'])
 
         self.text_columns = {}
@@ -105,9 +120,22 @@ class BasicDataCleaning(BaseEstimator, TransformerMixin):
         # See if we should fit TfidfVectorizer or not
         for key in X_df.columns:
 
+            if X_df[key].dtype == 'object' and self.column_descriptions.get(key, False) not in ['categorical', 'ignore', 'nlp']:
+                print('\n')
+                print('Encountered a column that is not marked as categorical, but is an "object" pandas type, which typically indicates a categorical column.')
+                print('The name of this columns is: "{}"'.format(key))
+                print('Some example features in this column are: {}'.format(list(X_df[key].head(5))))
+                print('If this is a categorical column, please mark it as `{}: "categorical"` as part of your column_descriptions'.format(key))
+                print('If this is not a categorical column, please consider converting its dtype before passing data into auto_ml')
+                print('\n')
+                warnings.warn('Consider marking the "{}" column as categorical'.format(key))
+
+            if self.transformed_column_descriptions.get(key) is None:
+                self.transformed_column_descriptions[key] = 'continuous'
+
             if key in self.text_columns:
                 X_df[key].fillna('nan', inplace=True)
-                text_col = X_df[key].astype(str, raise_on_error=False)
+                text_col = X_df[key].astype(unicode, errors='ignore')
                 self.text_columns[key].fit(text_col)
 
                 col_names = self.text_columns[key].get_feature_names()
@@ -127,6 +155,11 @@ class BasicDataCleaning(BaseEstimator, TransformerMixin):
 
     def transform(self, X, y=None):
 
+        ignore_none_fields = False
+        if self.get('transformed_column_descriptions', None) is not None:
+            ignore_none_fields = True
+        column_descriptions = self.get('transformed_column_descriptions', self.column_descriptions)
+
         # Convert input to DataFrame if we were given a list of dictionaries
         if isinstance(X, list):
             X = pd.DataFrame(X)
@@ -145,10 +178,12 @@ class BasicDataCleaning(BaseEstimator, TransformerMixin):
             dict_copy = {}
 
             for key, val in X.items():
-                col_desc = self.column_descriptions.get(key)
+                col_desc = column_descriptions.get(key, None)
 
-                if col_desc in (None, 'continuous', 'numerical', 'float', 'int'):
-                    dict_copy[key] = clean_val_nan_version(val)
+                if col_desc is None and ignore_none_fields is True:
+                    continue
+                elif col_desc in (None, 'continuous', 'numerical', 'float', 'int'):
+                    dict_copy[key] = clean_val_nan_version(key, val)
                 elif col_desc == 'date':
                     date_feature_dict = add_date_features_dict(X, key)
                     dict_copy.update(date_feature_dict)
@@ -189,38 +224,23 @@ class BasicDataCleaning(BaseEstimator, TransformerMixin):
 
         else:
             X.reset_index(drop=True, inplace=True)
+
             for key in X.columns:
-                col_desc = self.column_descriptions.get(key)
-                if col_desc == 'categorical':
+                col_desc = column_descriptions.get(key)
+
+                if col_desc is None and ignore_none_fields is True:
+                    continue
+
+                elif col_desc == 'categorical':
                     # We will handle categorical data later, one-hot-encoding it inside DataFrameVectorizer
                     pass
 
                 elif col_desc in (None, 'continuous', 'numerical', 'float', 'int'):
-                    # Make sure this isn't actually a date column, and if it is, give the user good information on how to fix this
-                    is_unmarked_date_col = False
-                    try:
-                        pd.to_datetime(X[key])
-                        is_unmarked_date_col = True
-                    except:
-                        pass
                     # For all of our numerical columns, try to turn all of these values into floats
                     # This function handles commas inside strings that represent numbers, and returns nan if we cannot turn this value into a float. nans are ignored in DataFrameVectorizer
                     try:
-                        X[key] = X[key].apply(clean_val_nan_version)
+                        X[key] = X[key].apply(lambda x: clean_val_nan_version(key, x))
                     except TypeError as e:
-                        if is_unmarked_date_col == True:
-                            print('\n\n\n')
-                            print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!')
-                            print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!')
-                            print('We have found a column that is not marked as a "date" column, but can be converted to dates.')
-                            print('The name of this column is:\n')
-                            print(key)
-                            print('\nThis column causes errors when we try to parse it as a numerical column.')
-                            print('If it is in fact a date column, please add this to the column_descriptions hash:')
-                            print('{' + key + ': date}')
-                            warnings.warn('UnmarkedDateColumn: Please mark the ' + key + ' column as being a date column in your column_descriptions dictionary.')
-                            print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!')
-                            print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n\n\n')
                         raise(e)
                     except UnicodeEncodeError as e:
                         print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!')
@@ -230,7 +250,7 @@ class BasicDataCleaning(BaseEstimator, TransformerMixin):
                         print(key)
                         print('The actual value that caused the issue is logged right above the exclamation points')
                         print('Please either mark this column as categorical, or clean up the values in this column')
-                        raise(e)
+                        # raise(e)
                         print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!')
                         print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!')
 
@@ -253,7 +273,7 @@ class BasicDataCleaning(BaseEstimator, TransformerMixin):
                     # col_names = ['nlp_' + key + '_' + str(word) for word in col_names]
 
                     X[key].fillna('nan', inplace=True)
-                    nlp_matrix = self.text_columns[key].transform(X[key].astype(str, raise_on_error=False))
+                    nlp_matrix = self.text_columns[key].transform(X[key].astype(unicode, errors='ignore'))
                     nlp_matrix = nlp_matrix.toarray()
 
                     text_df = pd.DataFrame(nlp_matrix)
@@ -306,10 +326,10 @@ def add_date_features_df(df, date_col):
     df.is_copy = False
 
     df[date_col] = pd.to_datetime(df[date_col])
-    df[date_col + '_day_of_week'] = df[date_col].apply(lambda x: x.weekday()).astype(int, raise_on_error=False)
+    df[date_col + '_day_of_week'] = df[date_col].apply(lambda x: x.weekday()).astype(int, errors='ignore')
 
     try:
-        df[date_col + '_hour'] = df[date_col].apply(lambda x: x.hour).astype(int, raise_on_error=False)
+        df[date_col + '_hour'] = df[date_col].apply(lambda x: x.hour).astype(int, errors='ignore')
 
         df[date_col + '_minutes_into_day'] = df[date_col].apply(lambda x: x.hour * 60 + x.minute)
     except AttributeError:
